--[[
  Auctioneer Advanced - Search UI - Searcher Pawn
  Version: 1.0.8 (Xit)
  Revision: $Id: SearcherPawn.lua 1.0.8 2009-03-30 11:00:00 Xit $
  URL: http://wow.curse.com/downloads/wow-addons/details/auc-advanced-searcher-pawn.aspx

  This is a plugin module for the SearchUI that assists in searching by evaluating items with Pawn

  License:
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program(see GPL.txt); if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Note:
    This AddOn's source code is specifically designed to work with
    World of Warcraft's interpreted AddOn system.
    You have an implicit license to use this AddOn with these facilities
    since that is its designated purpose as per:
    http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]
-- Create a new instance of our lib with our parent
local lib, parent, private = AucSearchUI.NewSearcher("Pawn")
if not lib then
  return
end
local print,decode,_,_,replicate,empty,_,_,_,debugPrint,fill = AucAdvanced.GetModuleLocals()
local get,set,default,Const = AucSearchUI.GetSearchLocals()

lib.tabname = "Pawn"

-- Set our defaults
default("search.pawn.scalename", "")
default("search.pawn.enchant", false)
default("search.pawn.canuse", false)
default("search.pawn.forcemh", false)
default("search.pawn.debug", false)

local scaletable = {}
local pclass = UnitClass("player")
local plevel = UnitLevel("player")

local function getScales()
  local scalecopy = {}
  scaletable = PawnGetAllScales()
  table.insert(scalecopy, { 1, "" })
  for i,j in pairs(scaletable) do
    table.insert(scalecopy, { i+1, j })
  end
  return scalecopy
end

-- This function is automatically called when we need to create our search generals
function lib:MakeGuiConfig(gui)
  -- Get our tab and populate it with our controls
  local id = gui:AddTab(lib.tabname, "Searchers")

  -- Add the help
  gui:AddSearcher("Pawn", "Search for items which Pawn considers an upgrade", 100)
  gui:AddHelp(id, "pawn searcher",
    "What does this searcher do?",
    "This searcher allows you to choose a scale that has been defined in the Pawn addon.  This searcher will then use Pawn to calculate values for items in the auction scan and only show items which are an upgrade for you.")

  gui:AddControl(id, "Header", 0, "Pawn Search Criteria")
  local last = gui:GetLast(id)
  gui:AddControl(id, "Selectbox",  0, 1, getScales(), "search.pawn.scalename", "ScaleName")
  local cont = gui:GetLast(id)
  gui:AddTip(id, "The Pawn Scale that will be used to determine the item value")
  last = cont
  gui:SetLast(id, last)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.enchant", "Check Enchant Value")
  last = gui:GetLast(id)
  gui:SetLast(id, last)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.canuse", "Useable Only")
  gui:AddTip(id, "Only items that your character can use.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.forcemh", "Force Main Hand")
  gui:AddTip(id, "Force the search to check one-hand weapons against main hand only.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.impoor", "Only What I Can Afford")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.28, 0, "search.pawn.buyout", "Use Buyout")
  gui:AddTip(id, "Use buyout instead of bid when checking auction prices.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.debug", "Show Debug Messages")
  gui:AddTip(id, "WARNING: Produces lots of output.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.28, 0, "search.pawn.debugfalse", "Only False")
  gui:AddTip(id, "Only show debug messages for items that do not match.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.28, 0, "search.pawn.debugtrue", "Only True")
  gui:AddTip(id, "Only show debug messages for items that match.")
end

local function Debug(Reason, Text)
  local showdbg = get("search.pawn.debug")
  local showfalse = get("search.pawn.debugfalse")
  local showtrue = get("search.pawn.debugtrue")

  if not showdbg then
    return
  end

  if showfalse and Reason ~= false then
    return
  end

  if showtrue and Reason ~= true then
    return
  end

  if DEFAULT_CHAT_FRAME then
    DEFAULT_CHAT_FRAME:AddMessage(tostring(Text))
  else
    message(tostring(Text))
  end
end

local function RGBPercToHex(r, g, b)
  r = r <= 1 and r >= 0 and r or 0
  g = g <= 1 and g >= 0 and g or 0
  b = b <= 1 and b >= 0 and b or 0
  return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

local function CanUse(link)

  MyScanningTooltip:ClearLines()
  MyScanningTooltip:SetHyperlink(link)
  local retval = true

  for i=1,MyScanningTooltip:NumLines() do
    --Check left side text to see if it is red, if so, can't use
    local mytext = getglobal("MyScanningTooltipTextLeft" .. i)
    local text = mytext:GetText()
    if (text) then
      local r,g,b,a = mytext:GetTextColor()
      local hex = RGBPercToHex(r,g,b)
      if (hex == "fe1f1f") then
        retval = false
      end
    end

    --Check right side text to see if it is red, if so, can't use
    local mytextr = getglobal("MyScanningTooltipTextRight" .. i)
    local textr = mytextr:GetText()
    if (textr) then
      local r,g,b,a = mytextr:GetTextColor()
      local hex = RGBPercToHex(r,g,b)
      if (hex == "fe1f1f") then
        retval = false
      end
    end
  end

  return retval
end

-- Convert between auctioneer equip positions and Pawn equip positions
local function ConvertSlot(pos,itype,subtype)
  local mainslot, otherslot

  local candual = 0
  local hasammo = 0
  local forcemh = get("search.pawn.forcemh")

  -- Limit one-handed weapon checks to main hand unless class can dual wield
  if (pclass == "Warrior" or pclass == "Hunter" and plevel >= 20) then
    candual = 1
  end
  if (pclass == "Shaman" and plevel >= 40) then
    candual = 1
  end
  if (pclass == "Rogue" and plevel >= 10) then
    candual = 1
  end
  if (pclass == "Death Knight") then
    candual = 1
  end

  if pos == 11 then  -- Finger
    mainslot = 11
    otherslot = 12
  elseif pos == 12 then -- Trinket
    mainslot = 13
    otherslot = 14
  elseif pos == 13 then -- One Hand Weapon
    mainslot = 16
    if candual and not forcemh then
      otherslot = 17
    end
  elseif pos == 14 then -- Shield
    mainslot = 17
  elseif pos == 15 then -- Ranged Right
    mainslot = 18
  elseif pos == 16 then -- Cloak
    mainslot = 15
  elseif pos == 17 then -- 2Handed
    mainslot = 16
    otherslot = 17
  elseif pos == 20 then -- Robe
    mainslot = 5
  elseif pos == 21 then -- Main Hand Weapon
    mainslot = 16
  elseif pos == 22 then -- Off hand
    if forcemh then
      mainslot = 16
    else
      mainslot = 17
    end
  elseif pos == 23 then -- Holdable
    if forcemh then
      mainslot = 16
    else
      mainslot = 17
    end
  elseif pos == 24 then -- Ammo
    mainslot = 0
  elseif pos == 25 then -- Thrown
    mainslot = 18
  elseif pos == 26 then -- Ranged
    mainslot = 18
  elseif (itype == "Armor") and ((subtype == "Totems") or (subtype == "Librams") or (subtype == "Idols") or (subtype == "Sigils")) then -- Relics
    mainslot = 18
  else
    mainslot = pos 
  end

  return mainslot, otherslot
end

function lib.Search(itemData)
  local link = itemData[Const.LINK]
  local itype = itemData[Const.ITYPE]
  local ipos  = itemData[Const.IEQUIP]
  local iname = itemData[Const.NAME]
  local subtype = itemData[Const.ISUB]
  local bid = itemData[Const.MINBID]
  local buyout = itemData[Const.BUYOUT]
  local debugfalse = get("search.pawn.debugfalse")
  local debugtrue = get("search.pawn.debugtrue")
  local pmoney = GetMoney()

  -- Realtime search support
  if type(ipos) == "string" then
    ipos = Const.InvTypes[ipos]
  end

  -- Did the user pick a scale
  local scalenum = get("search.pawn.scalename")
  local scalename = ""

  if not scalenum then
    Debug(false, "DBG: No scale defined, please pick a scale from the dropdown list.  If none exist you will need to define a scale in Pawn first.")
    return false, "No Scale Name"
  end

  -- Does the stored scale exist
  for i, j in pairs(scaletable) do
    if i == (scalenum - 1) then -- subtract 1, due to offset introduced in menu
      scalename = j
      break
    end
  end

  if scalename == "" then
    Debug(false, "DBG: No scale defined, please pick a scale from the dropdown list.  If none exist you will need to define a scale in Pawn first.")
    return false, "Scale not found or blank"
  end

  local scalefound = PawnDoesScaleExist(scalename)
  if not scalefound or scalefound ~= true then
    Debug(false, ("DBG: Error! Pawn cannot find scale [%s]"):format(scalename))
    return false, "Scale not found"
  end

  -- Does the item have a link
  if not link then
    Debug(false, "No Link")
    return false, "No link"
  end
  
  -- Don't show Ammo/thrown for non ammo classes
  if ((pclass ~= "Hunter" and pclass ~= "Rogue" and pclass ~= "Warrior") and (ipos == 24 or ipos == 25)) then
    Debug(false, ("DBG:%s Can't Use"):format(iname))
    return false, "Can't Use"
  end

  -- Only filter stuff that can be equipped
  if (not IsEquippableItem(link)) then
    Debug(false, ("DBG:%s Not Equippable"):format(iname))
    return false, "Not Equippable"
  end

  -- Check to see if we can use the item
  local canuse = get("search.pawn.canuse")
  local icanuse = CanUse(link)

  if canuse and icanuse ~= true then
    Debug(false, ("DBG:%s Can't use"):format(iname))
    return false, "Can't use"
  end

  -- Check to see if we can afford the item
  local impoor = get("search.pawn.impoor")
  local usebuyout = get("search.pawn.buyout")
  if (usebuyout and pmoney < buyout) or (impoor and pmoney < bid) then
    Debug(false, ("DBG:%s Can't afford aPrice[%d,%d] pMoney[%d]"):format(iname, bid, buyout, pmoney))
    return false, "Can't afford"
  end

  -----------------------------------------------------------------
  --Ok At this point we can use the item and possibly can afford it
  -----------------------------------------------------------------

  -- Get the signature of this item and find it's stats.
  local item = PawnGetItemData(link)

  if not item then
    Debug(false, ("DBG:%s Pawn has no data for this item."):format(iname))
    return false, "No Item"
  end

  local aVal, aValE = PawnGetSingleValueFromItem(item, scalename)

  if aVal == nil then
    aVal = 0
  end
  if aValE == nil then
    aValE = 0
  end

  local aStr = ("%0.2f"):format(aVal);
  local aStrE = ("(E)%0.2f"):format(aValE);

  -- Get value of equivalent slot(s).
  local mainslot, otherslot = ConvertSlot(ipos,itype,subtype)
  local mItem = PawnGetItemDataForInventorySlot(mainslot)

  -- If there is no item equipped in that slot, then any item is an upgrade
  if not mItem then
    Debug(true, ("DBG:(%d)%s aVal[%0.2f] > Empty Slot"):format(ipos,iname,aVal))
    --return aStr,nil,nil
    return "buy",nil,nil,aStr
  end

  -- Grab the values for the item equipped in the main slot
  local mVal, mValE = PawnGetSingleValueFromItem(mItem, scalename)
  local oItem = nil
  local oVal = 0
  local oValE = 0

  if (otherslot) then
    oItem = PawnGetItemDataForInventorySlot(otherslot)
    if oItem then
      oVal, oValE = PawnGetSingleValueFromItem(oItem, scalename)
    end
  end

  -- ensure we have valid values for comparisons

  if mVal == nil then
    mVal = 0
  end
  if mValE == nil then
    mValE = 0
  end
  if oVal == nil then
    oVal = 0
  end
  if oValE == nil then
    oValE = 0
  end

  -- compare 2-handed weapons against both main hand and off hand
  if ipos == 17 then
    local bVal = mVal + oVal
    if aVal > bVal then
      Debug(true, ("DBG:(%d)%s aVal[%0.2f] > bVal[%0.2f] (mVal[%0.2f] + oVal[%0.2f])"):format(ipos,iname,aVal,bVal,mVal,oVal))
      --return aStr,nil,nil
      return "buy",nil,nil,aStr
    end
  else 
    if aVal > mVal then 
      Debug(true, ("DBG:(%d)%s aVal[%0.2f] > mVal[%0.2f]"):format(ipos,iname,aVal,mVal))
      --return aStr,nil,nil
      return "buy",nil,nil,aStr
    end
    if otherslot and aVal > oVal then
      Debug(true, ("DBG:(%d)%s aVal[%0.2f] > oVal[%0.2f]"):format(ipos,iname,aVal,oVal))
      --return aStr,nil,nil
      return "buy",nil,nil,aStr
    end
  end

  local enchant = get("search.pawn.enchant")
  if (enchant) then
    -- compare 2-handed weapons against both main hand and off hand
    if ipos == 17 then
      local bValE = mValE + oValE
      if aValE > bValE then
        Debug(true, ("DBG:(%d)%s aValE[%0.2f] > bValE[%0.2f] (mValE[%0.2f] + oValE[%0.2f])"):format(ipos,iname,aValE,bValE,mValE,oValE))
        --return aStrE,nil,nil
        return "buy",nil,nil,aStr
      end
    else
      if aValE > mValE then
        Debug(true, ("DBG:(%d)%s aValE[%0.2f] > mValE[%0.2f]"):format(ipos,iname,aValE,mValE))
        --return aStrE,nil,nil
        return "buy",nil,nil,aStr
      end
      if otherslot and aValE > oValE then
        Debug(true, ("DBG:(%d)%s aValE[%0.2f] > oValE[%0.2f]"):format(ipos,iname,aValE,oValE))
        --return aStrE,nil,nil
        return "buy",nil,nil,aStr
      end
    end
  end

  Debug(false, ("DBG:(%d)%s aVal[%0.2f] too low"):format(ipos,iname,aVal))
  return false, "Pawn Value "..aVal.." too low"
end
